from scrapista.helpers.helpers import get_count, get_word_info
from bs4 import BeautifulSoup
from time import perf_counter
import concurrent.futures
import threading as th
import requests
import re
import datetime as dt


class GoodReadsScraper:
    
    def __init__(self,base_url="https://www.goodreads.com",headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"}):
        self.base_url = base_url
        pass


    @property
    def popular_quotes(self):
        """
            This property scrapes the most popular quotes on goodreads from 
            the first page of all the quotes
        """

        url = "https://www.goodreads.com/quotes?page=1"
        try:
            r = requests.get(url)
        except:
            return "Connection Error"

        soup = BeautifulSoup(r.content, "html.parser")
        quotes = soup.select(".quote")

        quotes_list = []
        
        for quote in quotes: 
            try: 
                like_tag = quote.select_one(".right .smallText")
                likes = int(get_count(like_tag.get_text("",strip=True)))
            except Exception as e:
                likes = "N/A"
                pass

            try:
                quote_tag = quote.select_one(".quoteText")
                for idx, quoteText in enumerate(quote_tag.stripped_strings): 
                    if idx == 0:
                        quote_text = quoteText

            except Exception as e:
                continue

            try: 
                by_tag = quote.select_one(".authorOrTitle")
                by = by_tag.get_text("",strip=True)
            except Exception as e:
                by = "Anonymous"
            

            quote_object = {
                "quote": quote_text,
                "by": by,
                "likes": likes
            }

            quotes_list.append(quote_object)
        
        return quotes_list


    def scrape_quotes_by_tag(self,tag,page=1):
        """
            This function expects a tag of a quote and a page number 
            and it will return a quote object in json form.
        """

        if page > 100: 
            raise(Exception("invalid page number: page number should be between 1-100"))

        url = f"https://www.goodreads.com/quotes/tag/{tag}?page={page}"
        try:
            r = requests.get(url)
        except Exception as e:
            return "Connection Error"

        soup = BeautifulSoup(r.content, "html.parser")
        quotes = soup.select(".quote")

        quotes_list = []
        
        for quote in quotes: 
            try: 
                like_tag = quote.select_one(".right .smallText")
                likes = int(get_count(like_tag.get_text("",strip=True)))
            except Exception as e:
                likes = "N/A"
                pass

            try:
                quote_tag = quote.select_one(".quoteText")
                for idx, quoteText in enumerate(quote_tag.stripped_strings): 
                    if idx == 0:
                        quote_text = quoteText

            except Exception as e:
                continue

            try: 
                by_tag = quote.select_one(".authorOrTitle")
                by = by_tag.get_text("",strip=True)
            except Exception as e:
                by = "Anonymous"
            

            quote_object = {
                "quote": quote_text,
                "by": by,
                "likes": likes
            }

            quotes_list.append(quote_object)
        
        return quotes_list


    def scrape_books_by_genre(self,genre): 
        
        url = "https://www.goodreads.com/genres/" + genre
        try:
            r = requests.get(url)
        except: 
            return "Connection Error"

        soup = BeautifulSoup(r.content, "html.parser")

        book_data_list = []

        wrappers = soup.select(".coverWrapper")

        links = []  
        for wrapper in wrappers: 
            try:
                link = wrapper.find("a")
            except: 
                raise(Exception("Genre is not a valid genre"))
            else: 
                links.append(link)

        images = []
        for link in links: 
            try: 
                images.append(link.find("img"))
            except Exception as e:
                pass
        
        for link, image in zip(links,images): 

            try: 
                book_url = self.base_url + link["href"]
            except Exception as e:
                book_url = "N/A"
                pass

            try: 
                book_title = image["alt"]
            except Exception as e: 
                if not book_url: 
                    continue
                book_title = "N/A"
                pass
                

            try:
                cover_url = image["src"]
            except Exception as e:
                cover_url = "N/A"
                pass

            book_object = {
                "title": book_title,
                "genre": genre,
                "url": book_url,
                "img_src": cover_url,
            }

            book_data_list.append(book_object)

        return book_data_list


    def scrape_book(self,url):
        """
            This function helps you scrape the data of a book by passing the 
            goodreads url of that book into the function
        """

        try:
            r = requests.get(url)
        except: 
            return "Connection Error"

        soup = BeautifulSoup(r.content, "html.parser")

        try: 
            title_tag = soup.select_one("#bookTitle")
            title = title_tag.get_text("",strip=True)
        except Exception as e: 
            title = None
            pass

        try: 
            book_cover_tag = soup.find(id="coverImage")
            book_cover = book_cover_tag["src"]
        except Exception as e:
            book_cover = None
            pass

        try:
            rating_tag = soup.find(itemprop="ratingValue")
            rating = float(rating_tag.get_text("",strip=True))
        except Exception as e:
            rating = None
            pass

        try:
            rating_count_tag = soup.find(itemprop="ratingCount")
            rating_count = get_count(rating_count_tag.get_text("",strip=True))
        except Exception as e:
            rating_count = None
            pass

        try:
           review_count_tag = soup.find(itemprop="reviewCount")
           review_count = get_count(review_count_tag.get_text("",strip=True))
        except Exception as e:
            review_count = None
            pass

            
        book_object = {
            "name": title,
            "rating(5)": rating,
            "rating_count": rating_count,
            "review_count": review_count,
            "img_src": book_cover
        }


        return book_object


    def async_scrape_books(self,urls):
        """
            This function runs the 'scrape_book' function asynchronously with 
            ThreadPoolExecutor.
        """

        book_list = []
        
        with concurrent.futures.ThreadPoolExecutor() as executor:
            for url in urls: 
                future = executor.submit(self.scrape_book,url=url)
                book_data = future.result()
                book_list.append(book_data)

        return book_list


start = perf_counter()
scraper = GoodReadsScraper()

adventure_urls = [movie["url"] for movie in scraper.scrape_books_by_genre("adventure")]
print(len(adventure_urls))
books_data = scraper.async_scrape_books(adventure_urls[:5])

print(books_data)
print(len(books_data))

end = perf_counter()

print(f"It took {round(end-start,2)} seconds(s)")
# start = perf_counter()

# gr = GoodReadsScraper()

"getting most popular quotes"

# quotes = gr.popular_quotes

# print(quotes)


"scraping books by their genre"

# book_list = gr.scrape_books_by_genre("mystery")

# print(book_list)
# print(len(book_list))


"scraping particular books by their url"

# book_data = gr.scrape_book("https://www.goodreads.com/book/show/2429135.The_Girl_with_the_Dragon_Tattoo")

# print(book_data)


"scraping multiple movies asynchronously"
"p.s. takes to much time at the moment"

# urls = [
#     "https://www.goodreads.com/book/show/2429135.The_Girl_with_the_Dragon_Tattoo",
#     "https://www.goodreads.com/book/show/52758545-the-black-coast",
#     "https://www.goodreads.com/book/show/54304115-the-girl-from-shadow-springs",
#     "https://www.goodreads.com/book/show/43263361-crown-of-bones"
# ]

# book_data_list = gr.async_scrape_books(urls)

# print(book_data_list)



"scrape a page of quotes of a tag"

# quote_list = gr.scrape_custom_quotes("life")

# print(quote_list)
# print(len(quote_list))


"scraping quotes by the custom values put into function"
"sync: 139.49s", "async: 271.80s"
# total_quotes = []
# for i in range(50):
#     with concurrent.futures.ThreadPoolExecutor() as executor:
#         future = executor.submit(gr.scrape_custom_quotes,tag="love",page=i)
#         quote_list = future.result()
#         total_quotes.extend(quote_list)

#     print(i, "done")

# print(len(total_quotes))



# headers = {
#     "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
# }



# end = perf_counter()

# print(f"In total it took {end-start:.2f} second(s)")


from bs4 import BeautifulSoup
from time import perf_counter
import concurrent.futures
import threading as th
import requests


class AmazonScraper:
    """
        This class has some methods that scrape the amazon website 
        and return you some data. 
    """
    def __init__(self,base_url="https://www.amazon.com",headers={"upgrade-insecure-requests": "1", "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"},params={"language": "en", "currency": "USD"}):
        self.base_url = base_url
        self.headers = headers
        self.params = params
        self.tracked = []
    
    
    def scrape_keyword(self,keyword,data_list=[]):
        """
            This method will return every product on amazon related to the 
            keyword you pass into the method
        """

        keyword_addition = keyword.replace(" ","+")
        url_keyword_addition = f"/s?k={keyword_addition}&ref=nb_sb_noss_2"
        url = self.base_url + url_keyword_addition

        try:
            r = requests.get(url,headers=self.headers,params=self.params)
        except:
            return "Connection Error"


        soup = BeautifulSoup(r.content, "html.parser")
        groups = soup.select(".s-result-item")

        for group in groups: 
            
            name_tag = group.select_one(".a-link-normal.a-text-normal")
            image = group.select_one(".s-image")
            stars = group.select_one(".a-icon-alt")
            price = group.select_one(".a-price-whole")
            price_symbol_tag = group.select_one(".a-price-symbol")

            try: 
                price_amount = price.get_text(strip=True).replace(",","")
                price_amount = float(price_amount)
            except: 
                continue

            try:
                price_symbol = price_symbol_tag.get_text("",strip=True)
                price_symbol = "(" + price_symbol + ")" 
            except:
                price_symbol = ""

            try: 
                source = image["src"]
            except: 
                source = "N/A"

            try: 
                star_amount = stars.get_text(strip=True)[:3].replace(",",".")
                star_amount = float(star_amount)
            except: 
                star_amount = "N/A"

            try:
                product_name = name_tag.get_text("|",strip=True)
            except: 
                product_name = "N/A"

            try: 
                product_url = self.base_url + name_tag["href"]
            except: 
                if product_name == "N/A":
                    continue
                product_url = "N/A"

            
            item_info = {
                "name": product_name,
                "price"+price_symbol: price_amount,
                "stars(5)": star_amount,
                "url": product_url,
                "img_src": source,
            }
            
            data_list.append(item_info)

        
        data_list = list(filter(lambda x: any(x.values()), data_list)) 

        return data_list


    def scrape_keywords(self,keywords,data_list=[]): 

        """
            Expects a list of items and it will scrape the data
            for each element in that list and return it in json form
        """

        if type(keywords) != list: 
            raise Exception("A list type is expected for 'keywords' argument")


        for keyword in keywords: 
            self.scrape_keyword(keyword,data_list)
        

        data_list = list(filter(lambda x: any(x.values()), data_list)) 

        return data_list


    def async_scrape_keywords(self,keywords,data_list=[]): 

        """
            This function is the exact same of the scrape_keyword function
            but this one runs with multiprocessing therefore it's much faster
        """

        if type(keywords) != list: 
            raise Exception("A list type is expected for 'keywords' argument")
        
        # alternative of ThreadExecutorPool
        """
        threads = [th.Thread(target=self.scrape_keyword,args=(keyword,data_list)) for keyword in keywords]

        for t in threads: 
            t.start()
        
        for t in threads: 
            t.join()
        """

        with concurrent.futures.ThreadPoolExecutor() as executor:
            for keyword in keywords: 
                future = executor.submit(self.scrape_keyword,keyword,data_list)


        data_list = list(filter(lambda x: any(x.values()), data_list)) 

        return data_list

    
    def track_item(self, url): 
        """
            Scrape all the instances of a single element
        """

        try:
            r = requests.get(url,headers=self.headers,params=self.params)
        except:
            return "Connection Error"

            
        soup = BeautifulSoup(r.content, "html.parser")

        title_tag = soup.select_one("#productTitle")
        stars_tag = soup.select_one(".a-icon.a-icon-star")
        price_tag = soup.find(id="priceblock_ourprice")

        if not price_tag: 
            price_tag = soup.find(id="priceblock_dealprice")
        note_tag = soup.find(id="vatMessage")


        try: 
            title = title_tag.get_text("|",strip=True)
        except Exception as e: 
            title = None

        try: 
            stars = stars_tag.get_text("|",strip=True)[:3].replace(",",".")
            stars = float(stars)
        except Exception as e: 
            stars = "N/A"
        
        try: 
            price = price_tag.get_text("|",strip=True)
        except Exception as e: 
            price = "N/A"

        try:
            price_int = price.replace(",","")[1:]
            price_int = float(price_int)
        except:
            "N/A"
        else:
            price = price_int

        try:
            currency = self.params["currency"]
            currency_addition = "(" + currency + ")"
        except:
            currency_addition = ""
        
        
        item_object = {
            "title": title,
            "stars(out of 5)": stars,
            "price"+currency_addition: price,
        }

        if note_tag: 
            item_object["note"] = note_tag.get_text(" ",strip=True)

        if url not in self.tracked: 
            self.tracked.append(url)


        return item_object


    def track_items(self,urls,data_list=[]):
        """
            Track all the items that are passed into the function as 
            list type. 
        """

        if type(urls) != list: 
            raise Exception("A list type is expected for 'urls' argument")

        for url in urls: 

            data = self.track_item(url)
            data_list.append(data)

        data_list = list(filter(lambda x: any(x.values()), data_list)) 

        return data_list


    def async_track_items(self,urls,data_list=[]):
        """
            This function is the asynchronous version of the 
            AmazonScraper.track_items function 
        """

        if type(urls) != list: 
            raise Exception("A list type is expected for 'urls' argument")


        with concurrent.futures.ThreadPoolExecutor() as executor:
            for url in urls: 
                future = executor.submit(self.track_item,url)
                data = future.result()
                data_list.append(data)


        data_list = list(filter(lambda x: any(x.values()), data_list)) 

        return data_list


start = perf_counter()

scraper = AmazonScraper()

url_data = scraper.scrape_keyword("basketball 1.2m hoop")


# data = scraper.scrape_keywords(["mouse","keyboard","monitor","yellow pencilcase"])

# print(data)
urls = [product["url"] for product in url_data]

data = scraper.async_track_items(urls[:5])

print(data)
print(len(data))

end = perf_counter()
print(f"It took {round(end-start,2)} seconds(s)")

# params = {
#     "language": "en", 
#     "currency": "USD"
# }

# headers = {
#     "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
# }


# start = perf_counter()
# scraper = AmazonScraper(headers,params=params)

"Sync Single keyword"

# data = scraper.scrape_keyword("airpod")

# print(data)

"Scraping multiple keywords"
# items = [
#     "pencil","cardboard","racket","paddle","book","headphone","water bottle","usb charger","book shelf","paper","markers","pencilcase","notebook","calculator"
# ]

"Sync scraping keywords"

# data_list = scraper.scrape_keywords(items)
# print(len(data_list))

"Async scraping keywords"

# data_list = scraper.async_scrape_keywords(items)
# print(len(data_list))



"Tracking single item"

# airpod = scraper.track_item("https://www.amazon.de/-/en/Lenovo-IdeaPad-Chromebook-WideView-Tablet/dp/B08CZXFKDP/?_encoding=UTF8&pd_rd_w=wofLG&pf_rd_p=07208ea4-5452-4c67-8f65-44901c0f68eb&pf_rd_r=0JJ8ZD05AEXFSZAQB3TS&pd_rd_r=8c4d6402-ff5e-4ee0-b158-13f872c81f7d&pd_rd_wg=BfT3H&ref_=pd_gw_ci_mcx_mr_hp_d")

# print(airpod)

"Tracking multiple items"

# urls = [
#     "https://www.amazon.de/-/en/Lenovo-IdeaPad-Chromebook-WideView-Tablet/dp/B08CZXFKDP/?_encoding=UTF8&pd_rd_w=wofLG&pf_rd_p=07208ea4-5452-4c67-8f65-44901c0f68eb&pf_rd_r=0JJ8ZD05AEXFSZAQB3TS&pd_rd_r=8c4d6402-ff5e-4ee0-b158-13f872c81f7d&pd_rd_wg=BfT3H&ref_=pd_gw_ci_mcx_mr_hp_d","https://www.amazon.de/Window-Battery-Charger-Supply-Karcher/dp/B07WCWHXRG/ref=sr_1_4_sspa?dchild=1&keywords=tuch&qid=1616610242&sr=8-4-spons&psc=1&spLa=ZW5jcnlwdGVkUXVhbGlmaWVyPUEyQUpJVk5aM0dLVExVJmVuY3J5cHRlZElkPUEwNzU0MTk3SThHNlk4M0xJQTNaJmVuY3J5cHRlZEFkSWQ9QTAyNzgxNzkyWkJBNlg4S0pPVkU0JndpZGdldE5hbWU9c3BfYXRmJmFjdGlvbj1jbGlja1JlZGlyZWN0JmRvTm90TG9nQ2xpY2s9dHJ1ZQ==","https://www.amazon.de/-/en/Heinz-Drache/dp/B014STN07I/ref=sr_1_12?dchild=1&keywords=tuch&qid=1616610258&sr=8-12","https://www.amazon.de/-/en/Lenovo-IdeaPad-Chromebook-WideView-Tablet/dp/B08CZXFKDP/?_encoding=UTF8&pd_rd_w=wofLG&pf_rd_p=07208ea4-5452-4c67-8f65-44901c0f68eb&pf_rd_r=0JJ8ZD05AEXFSZAQB3TS&pd_rd_r=8c4d6402-ff5e-4ee0-b158-13f872c81f7d&pd_rd_wg=BfT3H&ref_=pd_gw_ci_mcx_mr_hp_d","https://www.amazon.de/Window-Battery-Charger-Supply-Karcher/dp/B07WCWHXRG/ref=sr_1_4_sspa?dchild=1&keywords=tuch&qid=1616610242&sr=8-4-spons&psc=1&spLa=ZW5jcnlwdGVkUXVhbGlmaWVyPUEyQUpJVk5aM0dLVExVJmVuY3J5cHRlZElkPUEwNzU0MTk3SThHNlk4M0xJQTNaJmVuY3J5cHRlZEFkSWQ9QTAyNzgxNzkyWkJBNlg4S0pPVkU0JndpZGdldE5hbWU9c3BfYXRmJmFjdGlvbj1jbGlja1JlZGlyZWN0JmRvTm90TG9nQ2xpY2s9dHJ1ZQ==","https://www.amazon.de/-/en/Heinz-Drache/dp/B014STN07I/ref=sr_1_12?dchild=1&keywords=tuch&qid=1616610258&sr=8-12","https://www.amazon.de/-/en/Lenovo-IdeaPad-Chromebook-WideView-Tablet/dp/B08CZXFKDP/?_encoding=UTF8&pd_rd_w=wofLG&pf_rd_p=07208ea4-5452-4c67-8f65-44901c0f68eb&pf_rd_r=0JJ8ZD05AEXFSZAQB3TS&pd_rd_r=8c4d6402-ff5e-4ee0-b158-13f872c81f7d&pd_rd_wg=BfT3H&ref_=pd_gw_ci_mcx_mr_hp_d","https://www.amazon.de/Window-Battery-Charger-Supply-Karcher/dp/B07WCWHXRG/ref=sr_1_4_sspa?dchild=1&keywords=tuch&qid=1616610242&sr=8-4-spons&psc=1&spLa=ZW5jcnlwdGVkUXVhbGlmaWVyPUEyQUpJVk5aM0dLVExVJmVuY3J5cHRlZElkPUEwNzU0MTk3SThHNlk4M0xJQTNaJmVuY3J5cHRlZEFkSWQ9QTAyNzgxNzkyWkJBNlg4S0pPVkU0JndpZGdldE5hbWU9c3BfYXRmJmFjdGlvbj1jbGlja1JlZGlyZWN0JmRvTm90TG9nQ2xpY2s9dHJ1ZQ==","https://www.amazon.de/-/en/Heinz-Drache/dp/B014STN07I/ref=sr_1_12?dchild=1&keywords=tuch&qid=1616610258&sr=8-12","https://www.amazon.de/-/en/Lenovo-IdeaPad-Chromebook-WideView-Tablet/dp/B08CZXFKDP/?_encoding=UTF8&pd_rd_w=wofLG&pf_rd_p=07208ea4-5452-4c67-8f65-44901c0f68eb&pf_rd_r=0JJ8ZD05AEXFSZAQB3TS&pd_rd_r=8c4d6402-ff5e-4ee0-b158-13f872c81f7d&pd_rd_wg=BfT3H&ref_=pd_gw_ci_mcx_mr_hp_d","https://www.amazon.de/Window-Battery-Charger-Supply-Karcher/dp/B07WCWHXRG/ref=sr_1_4_sspa?dchild=1&keywords=tuch&qid=1616610242&sr=8-4-spons&psc=1&spLa=ZW5jcnlwdGVkUXVhbGlmaWVyPUEyQUpJVk5aM0dLVExVJmVuY3J5cHRlZElkPUEwNzU0MTk3SThHNlk4M0xJQTNaJmVuY3J5cHRlZEFkSWQ9QTAyNzgxNzkyWkJBNlg4S0pPVkU0JndpZGdldE5hbWU9c3BfYXRmJmFjdGlvbj1jbGlja1JlZGlyZWN0JmRvTm90TG9nQ2xpY2s9dHJ1ZQ==","https://www.amazon.de/-/en/Heinz-Drache/dp/B014STN07I/ref=sr_1_12?dchild=1&keywords=tuch&qid=1616610258&sr=8-12"
# ]

"Synchronously tracking multiple items"
# data_list = scraper.track_items(urls)
# print(data_list)

"Asynchronously tracking multiple items"
# data_list = scraper.async_track_items(urls)
# print(data_list)


"display tracked items"

# print(scraper.tracked)


# end = perf_counter()
# print(f"In total it took {end-start:.2f} seconds")

# scraper = AmazonScraper()
# print(scraper.track_item("https://www.amazon.de/-/en/23-8-inch-Full-all-one/dp/B089PJ5S5B/ref=sr_1_3?currency=USD&dchild=1&keywords=computer&qid=1617312928&sr=8-3"))



from scrapista.amazon import AmazonScraper
from scrapista.wikipedia import WikiScraper
import unittest 


headers = {
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
    "Accept-Language": "en-US,en;q=0.9,tr-TR;q=0.8,tr;q=0.7,en-GB;q=0.6"
}
class TestAmazonScraper(unittest.TestCase):
    
    def test_scrape_keyword(self):
        # you don't have to pass them in since they are already default
        scraper = AmazonScraper(headers=headers)
        data_list = scraper.scrape_keyword("pencil")

        self.assertEqual(type(data_list), list)
        self.assertGreater(len(data_list), 50)

    def test_track_item(self):
        scraper = AmazonScraper(headers=headers)
        item_info = scraper.track_item("https://www.amazon.de/-/en/23-8-inch-Full-all-one/dp/B089PJ5S5B/ref=sr_1_3?currency=USD&dchild=1&keywords=computer&qid=1617312928&sr=8-3")

        self.assertEqual(type(item_info), dict)
        

class TestWikiScraper(unittest.TestCase):

    def test_scrape_movie(self):
        scraper = WikiScraper()
        url = "https://en.wikipedia.org/wiki/Fight_Club"
        
        print(dir(scraper))
        #movie_data = scraper.scrape_movie(url)
        #print(movie_data)




if __name__ == "__main__":
    unittest.main()



from scrapista.helpers.helpers import put_year_limit,get_age,get_bdate,money_string_to_int
from bs4 import BeautifulSoup
from time import perf_counter
import concurrent.futures
import threading as th
import requests
import re
import datetime as dt



class WikiScraper:

    def __init__(self,base_url="https://en.wikipedia.org",headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"},):
        self.base_url = base_url


    @property
    def disney_movies(self):
        """
            This property returns the disney movies' urls 
            from the wikipedia page
        """
        r = requests.get("https://en.wikipedia.org/wiki/List_of_Walt_Disney_Pictures_films")
        soup = BeautifulSoup(r.content, "html.parser")

        tables = soup.select(".wikitable.sortable tbody")

        data_points = []
        data_list = []
        for table in tables: 
            rows = table.find_all("tr")
            for row in rows: 
                try: 
                    data_points.append(row.find_all("td")[1])
                except Exception as e:  
                    pass
            
        for data in data_points:
            disney_movie = {} 
            link_text = data.find("a")

            try:
                name = link_text.get_text("",strip=True)
            except Exception as e:
                pass
            else:
                disney_movie["name"] = name

            try: 
                link = link_text["href"]
            except Exception as e: 
                pass
            else: 
                if not(link[0] == "#"):
                    link = self.base_url + link 
                    disney_movie["url"] = link

            data_list.append(disney_movie)

        return data_list


    @property
    def highest_grossing_movies(self):
        """
            This function returns the urls of the highest grossing movies 
            in the world righ now.
        """

        r = requests.get("https://en.wikipedia.org/wiki/List_of_highest-grossing_films")
        soup = BeautifulSoup(r.content, "html.parser")

        table = soup.select_one(".wikitable.sortable").find("tbody")

        highest_grossing_movies = []

        for row in table.find_all("tr"):

            movie_object = {}
            link_tag = row.find_all("th")[0].find("a")

            try:
                movie_name = link_tag.get_text("",strip=True)
            except Exception as e:
                pass 
            else:
                movie_object["name"] = movie_name

            try:    
                link = link_tag["href"]
                link = self.base_url+link
            except Exception as e:
                pass
            else:
                movie_object["url"] = link

            if any(movie_object):
                highest_grossing_movies.append(movie_object)
        
        return highest_grossing_movies

    
    @property
    def most_important_people(self):
        """
            This function returns the most most important people of all
            times according to 'Times 100'.
        """

        url = "https://en.wikipedia.org/wiki/Time_100"
        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        gallery_boxes = soup.select(".gallerybox")
        gallery_texts = [gallery_box.select_one(".gallerytext") for gallery_box in gallery_boxes]

        people_list = []

        for text in gallery_texts:
            try:
                person_tag = text.find("a")
                name = person_tag.get_text("",strip=True)
                person_url = self.base_url + person_tag["href"]
            except Exception as e:
                pass
            else:
                person_object = {
                    "name": name,
                    "url": person_url,
                }

                people_list.append(person_object)


        return people_list


    def scrape_movie(self,url): 
        """
            You need to pass a wikipedia movie url into this function
            and it will return the info about that movie
        """

        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        info_box = soup.select_one(".infobox.vevent")

        if not info_box: 
            return

        for sup in info_box.find_all("sup"): 
            sup.decompose()

        movie_info = {}

        for idx, row in enumerate(info_box.find_all("tr")):

            if idx == 0: 
                movie_info["Name"] = row.get_text()
            
            if idx == 1: 
                continue

            if row.find("ul"): 
                header = row.find("th").get_text(" ", strip=True)
                items = [li.get_text(" ",strip=True).replace("\xa0"," ") for li in row.find("ul").find_all("li")]

                movie_info[header] = items

            else: 
                try: 
                    header = row.find("th").get_text(" ", strip=True)
                    data = row.find("td").get_text(" ", strip=True)
                except Exception as e: 
                    pass
                else: 
                    movie_info[header] = data

            
        # converting running time to integers
        try: 
            minute_number_pattern = re.compile("\d+")
            minutes_unit_pattern = re.compile("\w+")

            minutes = int(re.findall(minute_number_pattern,movie_info["Running time"])[0])
            minute_unit = re.findall(minutes_unit_pattern,movie_info["Running time"])[-1]
            movie_info[f"Running time({minute_unit})"] = minutes
        except Exception as e:
            pass
        else: 
            del movie_info["Running time"]
        

        # getting the gross of the movie
        try: 
            currency, gross = money_string_to_int(movie_info,"Box office")
            movie_info[f"Gross({currency})"] = gross
        except Exception as e:
            pass
        else:
            del movie_info["Box office"]

        try: 
            currency, budget = money_string_to_int(movie_info,"Budget")
            movie_info[f"Budget({currency})"] = budget
        except Exception as e:
            pass
        else:
            del movie_info["Budget"]


        return movie_info


    def async_scrape_movies(self,urls):
        """
            This function utlizies the 'scrape_movie_info' method 
            in multiple threads 
        """

        movies_urls = []

        with concurrent.futures.ThreadPoolExecutor() as executor: 
            for url in urls:
                future = executor.submit(self.scrape_movie,url)
                data = future.result()
                movies_urls.append(data)

        return movies_urls
            
        
    def scrape_notable_movies_by_year(self,year):
        """
            This function expects a year and scrapes the notable movies that 
            were released in that year.
        """

        put_year_limit(year,1920)

        try:
            year = int(year)
        except:
            raise(Exception("Please enter a valid year"))

        url = f"https://en.wikipedia.org/wiki/{year}_in_film"
        r = requests.get(url)

        soup = BeautifulSoup(r.content, "html.parser")

        lists = []

        for header in soup.find_all("h3"):
            ul = header.find_next("ul") 
            if ul:
                lists.append(ul)

        def scrape_movies_year_list(ul,movie_list):
            list_items = ul.find_all("li")

            for idx, item in enumerate(list_items):
                movie_object = {}
                try: 
                    movie_tag = item.find("i").find("a")
                    movie_title = movie_tag.get_text("",strip=True)
                    movie_url = movie_tag["href"]
                except Exception as e:
                    pass
                else:
                    movie_object["name"] = movie_title
                    movie_object["url"] = self.base_url + movie_url
                    if movie_object not in movie_list:
                        movie_list.append(movie_object)

        movie_list = []

        for ul in lists: 
            movie_data = scrape_movies_year_list(ul,movie_list)

        if year >= 2004: 

            table_body = soup.select_one(".wikitable.sortable tbody")

            for row in table_body.find_all("tr"):
                try: 
                    movie_tag = row.find("td").find("a")
                    movie_url = self.base_url + movie_tag["href"]
                    movie_title = movie_tag.get_text("",strip=True)
                except Exception as e:
                    pass
                else:
                    movie_object = {
                        "name": movie_title,
                        "url": movie_url
                    }
                    movie_list.append(movie_object)
                
            
        return movie_list


    def scrape_american_movies_by_year(self,year):
        """
            Scrapes the american movies by the year that was passed into
            the method.
        """

        put_year_limit(year,1920)

        url = f"https://en.wikipedia.org/wiki/List_of_American_films_of_" + str(year)
        r = requests.get(url)

        soup = BeautifulSoup(r.content, "html.parser")
        tables = soup.select(".wikitable")
        title_index = 0

        movie_list = []

        for table in tables: 
            head = table.find("thead")
            try: 
                for idx, headers in enumerate(head.find_all("th")):
                    if headers.get_text("",strip=True):
                        title_index = idx
            except Exception as e: 
                pass

            body = table.find("tbody")

            for row in body.find_all("tr"):
                try: 
                    element = row.find_all("td")[title_index]
                    movie_tag = element.find("a")
                    movie_url = self.base_url + movie_tag["href"]
                    movie_title = movie_tag.get_text("",strip=True)
                except Exception as e: 
                    pass
                else: 
                    movie_object = {
                        "name": movie_title,
                        "url": movie_url
                    }
                    movie_list.append(movie_object)


        return movie_list


    def scrape_person(self,url="",name=""): 
        """
            This function is similar to the scraping movie info function
            but you pass in a person page's url into it and it scrapes info 
            about that person
            infobox biography vcard
        """

        if not name and not url:
            return None

        if not url: 
            url_name = "_".join(name.split())
            url = self.base_url + "/wiki/" + url_name
        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        info_box = soup.select_one(".infobox.vcard")

        if not info_box: 
            print(f"{url} data not found")
            return

        # removing all the suffix and sup element from the DOM
        for sup in info_box.find_all("sup"): 
            sup.decompose()

        for br in info_box.find_all("br"):
            soup.i.string = " "
            br.replace_with(soup.i)

        for hidden in soup.find_all(style="display:none"): 
            hidden.decompose()

        try:
            info_box.find(class_="honorific-suffix").decompose()
        except Exception as e:
            pass


        person_info = {}

        if info_box.find("caption"): 
            person_info["Name"] = info_box.find("caption").get_text(" ",strip=True) 


        # scraping data
        for idx, row in enumerate(info_box.find_all("tr")):
            # getting the name of the person
            if idx == 0:
                if "Name" not in person_info.keys(): 
                    person_info["Name"] = row.get_text()

                        
            # if there is a list in the data get a list item
            if row.find("ul"): 
                try: 
                    header = row.find("th").get_text(" ", strip=True).replace("\xa0"," ")
                except Exception as e: 
                    header = "Miscellaneous"
                finally:
                    items = [li.get_text(" ",strip=True).replace("\xa0"," ").replace("\u200b","") for li in row.find("ul").find_all("li") if li.get_text(" ",strip=True)]
                if items:
                    person_info[header] = items

            # if there isn't any list scrape get items single 
            else: 
                try: 
                    header = row.find("th").string.replace("\xa0"," ")
                    data = row.find("td").get_text(" ",strip=True).replace("\xa0"," ").replace("\u200b","").replace("   "," ").replace("  ", " ").replace(" ,",",")
                except Exception as e: 
                    pass
                else: 
                    if data:
                        person_info[header] = data



        """Now let's get to convert and clean up the data a little"""


        # removing spaces in website url
        if "Website" in person_info.keys():
            person_info["Website"] = person_info["Website"].replace(" ","")

        # removing signature since it is not available
        try: 
            del person_info["Signature"]
        except Exception as e: 
            pass            

        # removing sports years team data
        try: 
            if person_info["Years"] == "Team":
                del person_info["Years"]
        except Exception as e:
            pass

        # removing awards if it is full list
        try: 
            if person_info["Awards"] == "Full list":
                del person_info["Awards"]
        except Exception as e:
            pass
                        

        try: 
            age = get_age(person_info,"Born")
        except Exception as e: 
            try:
                age = get_age("Date of birth")
                person_info["Age"] = age
            except Exception as e: 
                pass
        else: 
            person_info["Age"] = age


        try: 
            bdate = get_bdate(person_info,"Born")
        except Exception as e: 
            try: 
                person_info["Date of birth"] = person_info["Date of birth"].strip()
                if not person_info["Date of birth"]:
                    del person_info["Date of birth"]

            except Exception as e:
                pass
        else: 
            person_info["Birth Date"] = bdate
            person_info["Born"] = person_info["Born"].replace("   "," | ").replace(" ,",",").strip()

        
        # children number converted to an integer
        try: 
            children_count = re.findall("\d+",person_info["Children"])[0]
            person_info["Children"] = int(children_count)
        except Exception as e: 
            pass


        # converting spouses into a list object
        try: 
            bracket_pattern = "\(.*?\)"
            if type(person_info["Spouse(s)"]) == list:
                for idx,spouse in enumerate(person_info["Spouse(s)"]):
                    results = re.findall(bracket_pattern,spouse)
                    for result in results:
                        years_pattern = "\d+"
                        years = re.findall(years_pattern,result)
                        interval = "-".join(years)
                        present = ""
                        if len(results) == 1:
                            present = "-present"
                        person_info["Spouse(s)"][idx] = spouse.replace(result,"("+interval+present+")").replace("  ","").strip() 
                    
                spouse_list = person_info["Spouse(s)"]
            else:
                results = re.findall(bracket_pattern,person_info["Spouse(s)"])
                for idx,result in enumerate(results): 
                    years_pattern = "\d+"
                    years = re.findall(years_pattern,result)
                    interval = "-".join(years)
                    present = ""
                    if len(results) == 1:
                        present = "-present"
                    if idx==0:
                        person_info["Spouse(s)"] = person_info["Spouse(s)"].replace(result,"("+interval+")   ")
                    else:
                        person_info["Spouse(s)"] = person_info["Spouse(s)"].replace(result,"("+interval+present+")")
                
                person_info["Spouse(s)"] = person_info["Spouse(s)"].strip()
                spouse_list = person_info["Spouse(s)"].split("   ")
        except Exception as e: 
            pass
        else: 
            if spouse_list:
                person_info["Spouse(s)"] = spouse_list

        
        # cleaning the single data spouse
        try: 
            bracket_pattern = "\(.*\)"
            results = re.findall(bracket_pattern,person_info["Spouse"])
            for result in results: 
                years_pattern = "\d+"
                years = re.findall(years_pattern,result)
                interval = "-".join(years)
                present = ""
                if len(results) == 1:     
                    present = "-present"
                person_info["Spouse"] = person_info["Spouse"].replace(result,"("+interval+present+")")   
        except Exception as e: 
            pass
        else:
            person_info["Spouse"] = person_info["Spouse"].strip()


        # converting net worth integer with the currency
        try: 
            currency = person_info["Net worth"][:3]
            money_pattern = "\d+\.\d+"
            results = re.findall(money_pattern,person_info["Net worth"])
            if results: 
                if "billion" in person_info["Net worth"]: 
                    person_info["Net worth("+currency+")"] = 1_000_000_000 * float(results[0])
                elif "million" in person_info["Net worth"]: 
                    person_info["Net worth("+currency+")"] = 1_000_000 * float(results[0])
        except Exception as e:
            pass
        else: 
            del person_info["Net worth"]
        
        
        return person_info


    def async_scrape_people(self,urls="",names=""):
        """
            This function scrapes each person's info by threading. 
            It is much more efficient than running 'scrape_movie_info'
            method synchronously
        """ 

        people_info = []

        if names: 
            with concurrent.futures.ThreadPoolExecutor() as executor:
                for name in names:
                    future = executor.submit(self.scrape_person,name=name)
                    person_info = future.result()
                    people_info.append(person_info)
        else: 
            if not urls:
                return None
            with concurrent.futures.ThreadPoolExecutor() as executor:
                for url in urls:
                    future = executor.submit(self.scrape_person,url=url)
                    person_info = future.result()
                    people_info.append(person_info)

        return people_info


    def scrape_custom(self,url="",name=""):
        """
            This method is a vague one, you can pass in the url of an event, company, or even a place. This function will return the data on the info box if present, otherwise it will return None 
        """

        if not url and not name:
            return None

        if not url:
            url = "https://en.wikipedia.org/wiki/" + name
        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        info_box = soup.select_one(".infobox")

        if not info_box: 
            return

        for sup in info_box.find_all("sup"):
            sup.decompose()

        for br in info_box.find_all("br"):
            soup.i.string = " "
            br.replace_with(soup.i)

        for hidden in soup.find_all(style="display:none"): 
            hidden.decompose()

        custom_info = {}

        caption = info_box.find("caption")
        if caption: 
            custom_info["Name"] = caption.get_text(" ",strip=True).replace("\xa0", " ")


        for idx, row in enumerate(info_box.find_all("tr")):

            if idx == 0 and "Name" not in custom_info.keys(): 
                custom_info["Name"] = row.get_text("",strip=True).replace("\xa0", " ")
            
            if row.find("td") and row.find("ul"):
                try: 
                    header = row.find("th").get_text(" ", strip=True).replace("\xa0"," ")
                    items = [li.get_text(" ",strip=True).replace("\xa0"," ") for li in row.find("ul").find_all("li")]
                except Exception as e:
                    pass
                else:
                    custom_info[header] = items

            else: 
                try: 
                    go_to_url = row.select_one(".url")
                    header = row.find("th").get_text(" ", strip=True).replace("\xa0"," ").replace("•","").strip()
                    data = row.find("td").get_text(" ", strip=True).replace("\xa0"," ").replace("\ufeff", "").replace("•","").strip()
                    if go_to_url:
                        data = go_to_url.find("a")["href"]
                        
                except Exception as e: 
                    pass
                else: 
                    custom_info[header] = data


        # converting running time to integers
        try: 
            minute_number_pattern = re.compile("\d+")
            minutes_unit_pattern = re.compile("\w+")

            minutes = int(re.findall(minute_number_pattern,custom_info["Running time"])[0])
            minute_unit = re.findall(minutes_unit_pattern,custom_info["Running time"])[-1]
            movie_info[f"Running time({minute_unit})"] = minutes
        except Exception as e:
            pass
        else: 
            del custom_info["Running time"]
        

        # getting the gross of the movie
        try: 
            currency, gross = money_string_to_int(custom_info,"Box office")
            movie_info[f"Gross({currency})"] = gross
        except Exception as e:
            pass
        else:
            del custom_info["Box office"]

        try: 
            currency, budget = money_string_to_int(custom_info,"Budget")
            movie_info[f"Budget({currency})"] = budget
        except Exception as e:
            pass
        else:
            del custom_info["Budget"]

        if len(custom_info) < 2:
            return

        return custom_info



# ws = WikiScraper()
# start = perf_counter()

"disney movies property"
# print(ws.disney_movies_urls)

"scraping a movie info"

# person_info = ws.scrape_movie_info("https://en.wikipedia.org/wiki/Finding_Dory")

# print(person_info)

"scraping a person's info"

# data = ws.scrape_person_info(name="Jeff Bezos")
# data = ws.scrape_person_info(url="https://www.wikipedia.com/wiki/Elon_Musk")

# print(data)

"scraping the cast of a movie"

# cast = ws.scrape_movie_cast("https://en.wikipedia.org/wiki/Avengers:_Endgame")

# print(cast)

"scraping info of multiple people"

# people = [
#     "Elon Musk","Chris Evans","Gal Gadot","Jeff Bezos","Lionel Messi","Cristiano Ronaldo","Elon Musk","Chris Evans","Gal Gadot","Jeff Bezos","Lionel Messi","Cristiano Ronaldo","Elon Musk","Chris Evans","Gal Gadot","Jeff Bezos","Lionel Messi","Cristiano Ronaldo"
# ]


"scraping people info synchronously"

# people_info = []
# for person in people: 
#     person_info = ws.scrape_person_info(name=person)
#     people_info.append(person_info)

# print(people_info)
# print(len(people_info))



"scraping people info asynchronously"

# people_info = ws.async_scrape_person_info(names=people)

# print(people_info)
# print(len(people_info))


"highest grossing movies urls property"

# highest_grossing_movies = ws.highest_grossing_movies_urls

# print(highest_grossing_movies)


"requesting multiple data synchronously"

# all_movie_data = []

# movies = ws.highest_grossing_movies_urls

# for movie in movies: 
#     all_movie_data.append(ws.scrape_movie_info(movie))

# print(all_movie_data)
# print(len(all_movie_data))



"async movie info requests"


# all_movie_data = ws.async_scrape_movies_info(ws.highest_grossing_movies_urls)

# print(all_movie_data)
# print(len(all_movie_data))


"scraping notable movies by year they were released"

# for year in range(1920,2021):
#     with concurrent.futures.ThreadPoolExecutor() as executor: 
#         future = executor.submit(ws.scrape_notable_movies_by_year,year)
#         movie_list = future.result()
#         # print(movie_list)
#         print(f"{year}: {len(movie_list)}")


"scraping american movies by year"

# movie_data_list = ws.scrape_american_movies_by_year(1920)

# print(movie_data_list)
# print(len(movie_data_list))


"getting most important people"

# people = ws.most_important_people

# print(people)
# print(len(people))



# end = perf_counter()
# print(f"In total it took {round(end-start,2)} second(s)")




import requests
from bs4 import BeautifulSoup
import re
import datetime as dt

#  converting box office and budget into integer values that we got from wikipedia info box
def money_string_to_int(movie_info,key):
    
    currency = movie_info[key][0]
    movie_info[key] = movie_info[key].replace(currency,"")
    movie_info[key] = movie_info[key].strip()

    money_pattern = "(\d+-\d+|\d+\.\d+|\d+)"
    money_result = re.findall(money_pattern,movie_info[key])
    if not money_result:
        money_result = re.findall("\d+", movie_info[key])

    scale_pattern = "\w+"
    scale_result = re.findall(scale_pattern, movie_info[key])[-1]

    if scale_result == "billion": 
        scale = 1_000_000_00
    elif scale_result == "million": 
        scale = 1_000_000

    movies_monies = money_result

    for idx, money in enumerate(movies_monies):
        money = float(money)
        movies_monies[idx] = money * scale

    if len(movies_monies) == 1: 
        movies_monies = movies_monies[0]

    return (currency, movies_monies)


# this function raises exception when an invalid year is passed into a function as an argument
def put_year_limit(year,start,end=dt.datetime.now().year):
        
    if year > end or year < start:
        raise(BaseException(f"The year passed in should be between {start} and current year"))


# extracting the age from the 'born' key
# person_info is a dicionary and we will pass in the key we want to modify
def get_age(person_info,key):
    age_pattern = "\(age \d+\)"
    results = re.findall(age_pattern,person_info[key])
    result = results[0]
    person_info[key] = person_info[key].replace(result,"")
    if re.findall("^| ",person_info[key]):
        person_info[key] = person_info[key].replace("| ","")
    age = int(re.findall("\d+",result)[0])
    return age


# creating a datetime object of the person's birth time
# person_info is a dicionary and we will pass in the key we want to modify
def get_bdate(person_info,key):
    date_pattern = "\w+ \d+, \d+"
    result = re.findall(date_pattern,person_info[key])[0]
    person_info[key] = person_info[key].replace(result,"")
    bdate = dt.datetime.strptime(result,"%B %d, %Y")
    return bdate


# this function takes the review or critic count of a movie as a string from imdb
# then it converts eg. '7,805' into 7805 and returns the found int value
def get_count(str):
    str = str.replace(",","")
    results = re.findall("\d+", str)
    if len(results) == 1: 
        return int(results[0])
    else:
        return results



# this function scrapes and returns a word object 
# you have to pass the word and the headers in json formot user-agent key is required
def get_word_info(word, headers):
    
    r = requests.get("https://dictionary.cambridge.org/dictionary/english/"+word, headers=headers)

    soup = BeautifulSoup(r.content, "html.parser")

    word_info = {}
    word_info["word"] = word
    try:
        form = soup.select_one(".pos.dpos").get_text(" ",strip=True)
    except:
        form = None
    else:
        word_info["form"] = form 
    try:
        level = soup.select_one(".epp-xref.dxref").get_text(" ",strip=True)
    except:
        level = None
    else:
        word_info["level"] = level
    try:
        definition = soup.select_one(".def.ddef_d.db").get_text(" ",strip=True)
    except:
        definition = None
    else:
        word_info["definition"] = definition

    try:
        sentence_example_tags = soup.select(".eg.deg")
        sentence_examples = [sentence_example_tag.get_text(" ",strip=True) for sentence_example_tag in sentence_example_tags]
    except:
        sentence_examples = []
    else:
        word_info["sentence_examples"] = sentence_examples

    if not all(word_info.values()):
        return "No such word exists in the database"

    return word_info



from scrapista.helpers.helpers import money_string_to_int
from scrapista.helpers.helpers import put_year_limit
from scrapista.helpers.helpers import get_age
from scrapista.helpers.helpers import get_bdate
from scrapista.helpers.helpers import get_count
from scrapista.helpers.helpers import get_word_info


from setuptools import setup, find_packages

def get_readme():
    with open("README.md", encoding="utf-8") as fh:
        long_description = "\n" + fh.read()

    return long_description

VERSION = "0.2.0"
REQUIREMENTS = [
    'requests', 
    'beautifulsoup4'
]
DESCRIPTION = 'Scrape most popular websites easily'


# Setting up
setup(
    name="scrapista",
    version=VERSION,
    license='MIT',
    author="Alp Niksarlı",
    author_email="alp.niksarli@gmail.com",
    url='https://github.com/alpnix/scrapista',
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=get_readme(),
    packages=find_packages(),
    install_requires=REQUIREMENTS,
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    keywords=['python', 'scrape', 'amazon', 'wikipedia', 'imdb', 'goodreads', 'web', 'data mining', 'web scraping'],
    include_package_data=True,
)



from scrapista.helpers.helpers import get_count
from bs4 import BeautifulSoup
from time import perf_counter
import concurrent.futures
import threading as th
import datetime as dt
import requests
import re


class ImdbScraper:

    def __init__(self,base_url="https://www.imdb.com",headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"}):
        self.base_url = base_url
        pass

    @property
    def top_ranked_movies(self):
        
        r = requests.get("https://www.imdb.com/chart/top/")
        soup = BeautifulSoup(r.content, "html.parser")

        table = soup.select_one(".chart.full-width tbody")

        data_list = []

        for row in table.find_all("tr"):    

            image_tag = row.find_all("td")[0].find("a")
            img_src = self.base_url + image_tag["href"]

            title_tag = row.find_all("td")[1].find("a")
            title = title_tag.get_text(" ", strip=True)
            movie_src = self.base_url + title_tag["href"]

            rating_tag = row.find_all("td")[2].find("strong")
            rating = rating_tag.get_text(" ", strip=True)

            movie_object = {
                "name": title,
                "imdb_rating": float(rating),
                "url": movie_src,
                "img_src": img_src, 
            }

            data_list.append(movie_object)


        return data_list        


    @property
    def popular_movies(self):
        """
            This method returns the most popular movies by the current date. It returns 100 movies. 
        """
        url = "https://www.imdb.com/chart/moviemeter/?ref_=nv_mv_mpm"

        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        table_body = soup.select_one(".chart.full-width tbody")

        data_list = []

        for row in table_body.find_all("tr"):
            try: 
                a_tag = row.find_all("td")[1].find("a")
                link = self.base_url + a_tag["href"]
            except: 
                link = None
            else: 
                try:
                    name = a_tag.get_text("",strip=True)
                except: 
                    name = None

            data_list.append({"name": name, "url": link})

        return data_list    
        

    def scrape_movies_by_genre(self,genre):
        """
            With the help of this method you will get top 50 movies and tv shows by the genre passed into the function
        """

        url = f"https://www.imdb.com/search/title/?genres={genre}"

        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")

        movie_data = []

        movie_cards = [movie_item for movie_item in soup.select(".lister-item.mode-advanced")]
        try:
            movie_urls = [card.find("h3").find("a")["href"] for card in movie_cards]

            movie_urls = list(map(lambda x: self.base_url + x, movie_urls))
        
            movie_names = [card.find("h3").find("a").get_text("",strip=True) for card in movie_cards]
        except:
            return []
        else: 
            for i in range(len(movie_names)):
                movie_object = {"name": movie_names[i],"url": movie_urls[i]}
                if movie_object not in movie_data:
                    movie_data.append(movie_object)

        return movie_data


    def scrape_movie(self,url):
        
        r = requests.get(url)
        soup = BeautifulSoup(r.content, "html.parser")


        title = soup.find("h1").get_text("",strip=True)
        
        try:
            rating_count = int(soup.find(itemprop="ratingCount").get_text("", strip=True).replace(",",""))
        except:
            rating_count = "N/A"

        try:
            rating_value = float(soup.find(itemprop="ratingValue").get_text("",strip=True).replace(",",""))
        except:
            rating_value = "N/A"

        try:
            running_time = soup.select_one(".title_wrapper .subtext time").get_text("",strip=True)
        except:
            running_time = "N/A"

        def get_running_minutes(str):
            minutes = 0
            hour_pattern = "\d+h"
            minute_pattern = "\d+min"
            try: 
                hours = re.findall(hour_pattern,str)[0]
                minutes += int(hours[0]) * 60
            except Exception as e: 
                pass
            finally:
                try:
                    minutes_min = re.findall(minute_pattern,str)[0]
                    minutes += int(re.findall("\d+",minutes_min)[0])
                    return minutes
                except Exception as e: 
                    return minutes

        running_time_int = get_running_minutes(running_time)

        try:
            *genre_tags, release_tag = soup.select(".title_wrapper .subtext a")

            genres = [genre.get_text("",strip=True) for genre in genre_tags]

            release = release_tag.get_text("",strip=True)
        except:
            genres = []
            release = "N/A"

        restriction = list(soup.select_one(".title_wrapper .subtext").stripped_strings)[0]
        if len(restriction) > 4: 
            restriction = "R?"
            
        try:
            trailer = self.base_url + soup.find("a", class_=["video-modal"])["href"]
        except:
            trailer = self.base_url + "/404.html"

        try:
            image_source = soup.select_one(".poster img")["src"]
        except:
            image_source = self.base_url + "/404.html"

        try:
            metascore = float(soup.select_one(".metacriticScore span").get_text("",strip=True))
        except: 
            metascore = "N/A"

        try:
            review_tag, critic_tag = soup.select(".titleReviewBarItem.titleReviewbarItemBorder .subText a")
            review_count = get_count(review_tag.get_text("",strip=True))
            critic_count = get_count(critic_tag.get_text("",strip=True))
        except: 
            review_count = "-"
            critic_count = "-"


        cast_table = soup.select_one("table.cast_list")

        cast_list = []
        for row in cast_table.find_all("tr"):
            try: 
                cast_list.append(row.find_all("td")[1].get_text("",strip=True))
            except Exception as e:
                pass
            
        
        summary = soup.select_one(".plot_summary")


        movie_object = {
            "name": title,
            "rating_count": rating_count,
            "rating_value": rating_value,
            "running_time(min)": running_time_int,
            "genres": genres,
            "release": release,
            "restriction": restriction,
            "trailer": trailer,
            "image_source": image_source,
            "metascore": metascore,
            "review_count": review_count,
            "critic_count": critic_count,
        }

        for row in summary.find_all("div"):
            try:
                header = row.find("h4").get_text("",strip=True)[:-1]
                if header in ["Writer","Writers"]:
                    header = "writer(s)"
                elif header in ["Director","Directors"]:
                    header = "director(s)"
                elif header in ["Stars","Star"]: 
                    header = "star(s)"
                a_tags = [a_tags for a_tags in row.find_all("a",href=re.compile("^/name"))]
                data = [a_tag.get_text("",strip=True) for a_tag in a_tags]
            except Exception as e:
                pass
            else:
                movie_object[header] = data

        movie_object["cast"] = cast_list


        return movie_object
    

    def async_scrape_movies(self,urls,checkpoints=False):
        """
            This function scrapes each movie asynchronously
            instead of scraping them one by one
        """

        if checkpoints == 0 or (type(checkpoints) == bool and checkpoints == True):
            raise(Exception("checkpoint should be a positive integer"))

        all_movie_list = []

        with concurrent.futures.ThreadPoolExecutor() as executor:
            for idx,url in enumerate(urls): 
                if checkpoints and idx % checkpoints == 0:
                    print("Getting data... {}/{}".format(idx,len(urls)))
                future = executor.submit(self.scrape_movie, url)
                movie_info = future.result()
                all_movie_list.append(movie_info)

        return all_movie_list


    def scrape_actors_by_bdate(self,date=""):
        """
            This method returns the actors that were born today or optionally 
            you can pass a day as an argument and get the actors that were born
            in that day. P.s. date format is month-day eg '05-24'
        """

        if not date: 
            now = dt.datetime.now()
            date = f"{now.month}-{now.day}"

        url = "https://www.imdb.com/search/name/?birth_monthday=" + date

        r = requests.get(url)

        soup = BeautifulSoup(r.content, "html.parser")

        actor_cards = [card for card in soup.select(".lister-item.mode-detail")]
        actor_tags = [card.find("h3").find("a") for card in actor_cards]

        try:
            actor_names = [tag.get_text("",strip=True) for tag in actor_tags]

        except Exception as e:
            pass


        if actor_names == []:
            raise(Exception("date format is month-day eg. 05-24"))

        

        return actor_names



scraper = ImdbScraper()

# movie_urls = [movie["url"] for movie in scraper.top_ranked_movies]
# print(movie_urls[59])
# movie_info_list = scraper.async_scrape_movies(movie_urls,checkpoints=1)

# print(len(movie_info_list))

# with open("results.txt","w",encoding="utf-8") as f:
#     f.write(str(movie_info_list))



# scraper = ImdbScraper()
# start = perf_counter()


"getting top 250 movies"

# movie_list = scraper.top_ranked_movies_data

# print(movie_list)

"scrape a single movie"

# movie_index = 65
# queue: 138,193,205,216,232

# movie_info = scraper.scrape_movie("https://www.imdb.com/title/tt2737304/?ref_=fn_al_tt_1/")

# print(movie_info)

"async scraping multiple movies"
"this method is 2.5 times faster than running them synchronously"

# movie_urls = [movie["movie_url"] for movie in movie_list]

# all_movie_data = scraper.async_scrape_movies(movie_urls)

# print(all_movie_data)


"getting 50 movies urls by their genres"

# horror_urls = scraper.scrape_movies_by_genre("adventure")

# print(horror_urls)


"scraping the names and the links of most popular movies"

# popular_movies = scraper.most_popular_movies_data

# print(popular_movies)


"scraping actors that were born today"

# born_today = scraper.scrape_actors_by_birthdate()

# print(born_today)




# end = perf_counter()
# print(f"In total it took {round(end-start,2)} second(s)")